/*  Jcurzez - A free Java implementation of curses
 *  Copyright (C) 2001-2007  Thomas Girard <thomas.g.girard@free.fr>
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.nongnu.savannah.jcurzez;

import atomicsets.OrderedLock;

/**
 * An area. Unlike {@link jcurzez.Screen Screen}, <code>linewrap</code> and
 * <code>scroll</code> can be specified on creation.
 * <p>
 * <em>An area by default shares the buffer of its parent.</em>
 *
 * @author Thomas Girard
 * @version 1.0.0
 */
public class Area extends AbstractWindow {
    
	// aj - remove to avoid constructing shared buffer without proper aliasing
	/**
     * Creates a new <code>Area</code> using specified
     * {@link jcurzez.Rectangle Rectangle}. The parent of this area will be
     * the screen, and the buffer of this area will be shared with the
     * screen.
     *
     * @param  rectangle the bounds for this area.
     */
//    protected Area(final Rectangle rectangle) {
//    	this(rectangle, true, false, true);
//    }

    /**
     * Creates a new <code>Area</code> with specified bounds and
     * properties. The parent of this area will be the screen.
     *
     * @param  rectangle the bounds for this area.
     * @param  linewrap <code>true</code> to allow line wrapping.
     * @param  scroll <code>true</code> to allow autoscrolling.
     * @param  share <code>true</code> to share the buffer of the parent.
     */
//    protected Area(final Rectangle rectangle,
//    			final boolean linewrap,
//    			final boolean scroll,
//    			final boolean share) {
//    	this(Screen.getScreen(), rectangle, linewrap, scroll, share);
//    }

    /**
	 * Constructs a new <code>Area</code> with a parent, specified bounds and properties.
	 * @param parent  the parent of this area.
	 * @param rectangle  the bounds for this area.
	 * @param linewrap  <code>true</code> to allow line wrapping.
	 * @param scroll  <code>true</code> to allow autoscrolling.
	 * @param share  <code>true</code> to share the buffer of the parent.
	 */
	protected Area(final AbstractWindow parent, final Rectangle rectangle,
			final boolean linewrap, final boolean scroll, final boolean share) {
		this(parent, rectangle, linewrap, scroll, share, new OrderedLock());
	}

	/**
     * Constructs a new <code>Area</code> with a parent, specified
     * bounds and properties.
     *
     * @param  parent the parent of this area.
     * @param  rectangle the bounds for this area.
     * @param  linewrap <code>true</code> to allow line wrapping.
     * @param  scroll <code>true</code> to allow autoscrolling.
     * @param  share <code>true</code> to share the buffer of the parent.
     */
    protected Area(final AbstractWindow parent /*B<=this.B*/,
    			final Rectangle rectangle,
    			final boolean linewrap,
    			final boolean scroll,
    			final boolean share, OrderedLock B) {
//    	super(parent, new Cursor/*C=this.B*/(rectangle, 0, 0), linewrap, scroll, share);
    	super(parent, rectangle, linewrap, scroll, share, B);
    }

    /**
     * Constructs a new <code>Area</code> with a parent, specified
     * bounds and properties.
     *
     * @param  parent the parent of this area.
     * @param  rectangle the bounds for this area.
     * @param  linewrap <code>true</code> to allow line wrapping.
     * @param  scroll <code>true</code> to allow autoscrolling.
     */
//    public Area(final AbstractWindow parent,
//    			final Rectangle rectangle,
//    			final boolean linewrap,
//    			final boolean scroll) {
//    	this(parent, rectangle, linewrap, scroll, true);
//    }

    /**
     * Emits a beep.
     */
    public void beep_internal() {
    	Screen.peer.beep();
    }

	/**
	 * Emits a beep.
	 */
	public void beep() {
		synchronized (this.$lock_B) {
			Screen.peer.beep();
		}
	}

    /**
     * Updates the area.
     */
    public void update_internal() {
    	// Hack this to get AJ to allow desired concurrency.  
    	Screen.peer.update();
    	//this.parent.update();
    }

	/**
	 * Updates the area.
	 */
	public void update() {
		synchronized (this.$lock_B) {
			// Hack this to get AJ to allow desired concurrency.  
			Screen.peer.update();
			//this.parent.update();
		}
	}

    /**
     * Invokes parent's <code>printCell</code> method.
     */
    public void printCell_internal(final int x, final int y, final Cell cell) {
    	Screen.peer.printCell(x, y, cell);
    }

	/**
	 * Invokes parent's <code>printCell</code> method.
	 */
	public void printCell(final int x, final int y, final Cell cell) {
		synchronized (this.$lock_B) {
			Screen.peer.printCell(x, y, cell);
		}
	}

    /**
     * Invokes parent's <code>gotoXY</code> method.
     */
    public void gotoXY_internal(final int x, final int y) {
    	Screen.peer.gotoXY(x, y);
    }

	/**
	 * Invokes parent's <code>gotoXY</code> method.
	 */
	public void gotoXY(final int x, final int y) {
		synchronized (this.$lock_B) {
			Screen.peer.gotoXY(x, y);
		}
	}
}
