/*  Jcurzez - A free Java implementation of curses
 *  Copyright (C) 2001, 2006, 2008 Thomas Girard <thomas.g.girard@free.fr>
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
 */
package org.nongnu.savannah.jcurzez;

/**
 * Text attribute: <b>bold</b>, <u>underline</u>, and so on.
 * <p>
 * There are several predefined attributes:
 * <ul>
 *   <li>{@link #NORMAL normal attribute}</li>
 *   <li>{@link #BLINK <blink>blink</blink> attribute}</li>
 *   <li>{@link #BOLD <b>bold</b> attribute}</li>
 *   <li>{@link #REVERSE reverse attribute}</li>
 *   <li>{@link #UNDERLINE <u>underline</u> attribute}</li>
 * </ul>
 * <p>
 * Attributes can be combined (using {@link #and and()} or
 * {@link #andNot andNot()}). Thus, if we want to use a bold and underline
 * attribute, we create this compound attribute as shown below:
 * 
 * <pre>
 * Attribute boldAndUnderline = Attribute.BOLD.and(Attribute.UNDERLINE);
 * </pre>
 * 
 * @author Thomas Girard
 * @version 1.0.0
 */
public final class Attribute {

    /** The value of the attribute. */
    private final byte value;

    /**
     * Creates a new attribute with value <code>value</code>.
     *
     * @param value
     *            the value of the attribute.
     */
    private Attribute(final byte value) {
        this.value = value;
    }

    // Values used for attributes. When updating Java dependency to 5, this
    // could use enum.
    private static final byte NORMAL_BYTE = (byte) 0;
    private static final byte BLINK_BYTE = (byte) (1 << 0);
    private static final byte BOLD_BYTE = (byte) (1 << 1);
    private static final byte REVERSE_BYTE = (byte) (1 << 2);
    private static final byte UNDERLINE_BYTE = (byte) (1 << 3);

    /** NORMAL attribute. */
    public static final Attribute NORMAL = new Attribute(NORMAL_BYTE);

    /** BLINK attribute. */
    public static final Attribute BLINK = new Attribute(BLINK_BYTE);

    /** BOLD attribute. */
    public static final Attribute BOLD = new Attribute(BOLD_BYTE);

    /** REVERSE attribute. */
    public static final Attribute REVERSE = new Attribute(REVERSE_BYTE);

    /** UNDERLINE attribute. */
    public static final Attribute UNDERLINE = new Attribute(UNDERLINE_BYTE);

    /**
     * Checks whether <code>this</code> has attribute <code>a</code>.
     * 
     * @param a
     *            the attribute we want to check <code>this</code> against.
     * @return <code>true</code> if <code>this</code> has attribute
     *         <code>a</code> on.
     */
    public boolean hasAttribute(final Attribute a) {
        if (a == null) {
            throw new NullPointerException("attribute cannot be null");
        }

        return (this.value & a.value) != 0;
    }

    /**
     * Creates a new <code>Attribute</code> that combines <code>a</code> and
     * <code>this</code>.
     * 
     * @param a
     *            the attribute to combine with <code>this</code>.
     * @return the newly created attribute, or <code>this</code> if
     *         <code>a</code> attribute was already set for <code>this</code>.
     */
    public Attribute and(final Attribute a) {
        // Minimize object creation.
        if (hasAttribute(a)) {
            return this;
        }

        return new Attribute((byte) (this.value | a.value));
    }

    /**
     * Create a new attribute that has attribute <code>a</code> off.
     * 
     * @param a
     *            the attribute to unset from <code>this</code>.
     * @return the newly created attribute, or <code>this</code> if
     *         <code>a</code> was not set in it.
     */
    public Attribute andNot(final Attribute a) {
        // Minimize object creation.
        if (!hasAttribute(a)) {
            return this;
        }

        return new Attribute((byte) (this.value & ~a.value));
    }

    /**
     * Tests equality of <code>this</code> with <code>o</code>.
     * 
     * @param o
     *            the object we want to test.
     * @return <code>true</code> if <code>this</code> and <code>o</code>
     *         are equal.
     */
    public boolean equals(final Object o) {
        if (o == this) {
            return true;
        }

        if (o instanceof Attribute) {
            return this.value == ((Attribute) o).value;
        }

        return false;
    }

    /**
     * Returns the hashcode of this attribute. Since having same {@link #value}
     * means attributes are equal, this is the the hashcode.
     * 
     * @return {@link #value}.
     */
    public int hashCode() {
        return (int) this.value;
    }

    /**
     * Returns a string representation of <code>this</code>, in the form [
     * bold underline ].
     * 
     * @return a string representation of <code>this</code>.
     */
    public String toString() {
        boolean hasOne = false;

        StringBuffer sb = new StringBuffer("");

        if (hasAttribute(BLINK)) {
            sb.append("blink ");
            hasOne = true;
        }

        if (hasAttribute(BOLD)) {
            sb.append("bold ");
            hasOne = true;
        }

        if (hasAttribute(REVERSE)) {
            sb.append("reverse ");
            hasOne = true;
        }

        if (hasAttribute(UNDERLINE)) {
            sb.append("underline ");
            hasOne = true;
        }

        return (hasOne) ? sb.toString() : sb.append("normal ").toString();
    }

    /**
     * Returns the value describing <code>this</code>.
     * 
     * @return {@link #value}.
     */
    public byte getValue() {
        return this.value;
    }
}
