/*  Jcurzez - A free Java implementation of curses
 *  Copyright (C) 2001, 2006  Thomas G. Girard <thomas.g.girard@free.fr>
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.nongnu.savannah.jcurzez;

import atomicsets.OrderedLock;

/**
 * A <code>Pen</code> contains three attributes:
 * <ul>
 * <li>a {@link #foreground foreground color}</li>
 * <li>a {@link #background background color}</li>
 * <li>an {@link #attribute attribute}</li>
 * </ul>
 *
 * @author Thomas Girard
 * @version 1.0.0
 */
public class Pen implements atomicsets.Atomic {
    /*atomicset(P)*/
	
	public final OrderedLock getLockForP() {
		return $lock_P;
	}

	public final OrderedLock getLock() {
		return this.getLockForP();
	}

	/** Foreground color for this <code>Pen</code>. */
    /*atomic(P)*/protected ForegroundColor foreground;

    /** Background color for this <code>Pen</code>. */
    /*atomic(P)*/protected BackgroundColor background;

    /** Attribute for this <code>Pen</code>. */
    /*atomic(P)*/protected Attribute attribute;
    
    
    // aj remove
    //AbstractWindow aw;

    protected Pen(Pen p) {
		this(p, new OrderedLock());
	}

	// aj allow reconstruction with alias
    protected Pen(/*unitfor(P)*/ Pen p, OrderedLock P){
		this (p.getForegroundColor_internal(),p.getBackgroundColor_internal(),p.getAttribute_internal(), P);
		synchronized (p.getLockForP()) {
		}
	}
    
    /**
	 * Creates a new pen with defaults values for foreground color, background color and attribute.
	 */
	public Pen() {
		this(new OrderedLock());
	}

	/**
     * Creates a new pen with defaults values for
     * foreground color, background color and attribute.
     */
    public Pen(OrderedLock P) {
	this(ForegroundColor.WHITE, BackgroundColor.BLACK, Attribute.NORMAL, P);
    }

    /**
	 * Creates a new pen with specified foreground, background and attribute.
	 * @param foreground  the foreground color.
	 * @param background  the background color.
	 * @param attribute  the attribute fro this pen.
	 */
	public Pen(ForegroundColor foreground, BackgroundColor background,
			Attribute attribute) {
		this(foreground, background, attribute, new OrderedLock());
	}

	/**
     * Creates a new pen with specified foreground, background
     * and attribute.
     *
     * @param     foreground the foreground color.
     * @param     background the background color.
     * @param     attribute the attribute fro this pen.
     */
    public Pen(ForegroundColor foreground,
	       BackgroundColor background,
	       Attribute attribute, OrderedLock P) {
	$lock_P = P;
	this.foreground = foreground;
	this.background = background;
	this.attribute = attribute;
    }

    /**
     * Checks whether <code>this</code> is equal to <code>o</code>.
     *
     * @param     o the object to compare <code>this</code> with.
     * @return    <code>true</true> if <code>this</code> is equal to
     *            <code>o</code>.
     */
    public boolean equals_internal(Object o) {
	if (this == o)
	    return true;
	if (!(o instanceof Pen))
	    return false;
	Pen pen = (Pen) o;
	return (pen.getForegroundColor() == foreground &&
		pen.getBackgroundColor() == background &&
		pen.getAttribute().equals(attribute));
    }

	/**
	 * Checks whether <code>this</code> is equal to <code>o</code>.
	 * @param o  the object to compare <code>this</code> with.
	 * @return     <code>true</true> if <code>this</code> is equal to <code>o</code>.
	 */
	public boolean equals(Object o) {
		if (this == o)
		    return true;
		if (!(o instanceof Pen))
		    return false;
		Pen pen = (Pen) o;
		synchronized (this.$lock_P) {
			return (pen.getForegroundColor() == foreground &&
				pen.getBackgroundColor() == background &&
				pen.getAttribute().equals(attribute));
		}
	}

    /**
     * Changes the background color.
     *
     * @param     background the new background color.
     */
    public void setBackgroundColor_internal(BackgroundColor background) {
//    	synchronized(aw){
	this.background = background;
//    	}
    }

	/**
	 * Changes the background color.
	 * @param background  the new background color.
	 */
	public void setBackgroundColor(BackgroundColor background) {
		synchronized (this.$lock_P) {
			//    	synchronized(aw){
			this.background = background;
//    	}
		}
	}

    /**
     * Changes the foreground color.
     *
     * @param     foreground the new foreground color.
     */
    public void setForegroundColor_internal(ForegroundColor foreground) {
//    	synchronized(aw){
	this.foreground = foreground;
//    	}
    }

	/**
	 * Changes the foreground color.
	 * @param foreground  the new foreground color.
	 */
	public void setForegroundColor(ForegroundColor foreground) {
		synchronized (this.$lock_P) {
			//    	synchronized(aw){
			this.foreground = foreground;
//    	}
		}
	}

    /**
     * Changes the attribute.
     *
     * @param     attribute the new attribute.
     */
    public void setAttribute_internal(Attribute attribute) {
//    	synchronized(aw){
	this.attribute = attribute;
//    	}
    }

	/**
	 * Changes the attribute.
	 * @param attribute  the new attribute.
	 */
	public void setAttribute(Attribute attribute) {
		synchronized (this.$lock_P) {
			//    	synchronized(aw){
			this.attribute = attribute;
//    	}
		}
	}

    /**
     * Retrieves the foreground color.
     *
     * @return    the foreground color.
     */
    public ForegroundColor getForegroundColor_internal() {
	return foreground;
    }

	/**
	 * Retrieves the foreground color.
	 * @return     the foreground color.
	 */
	public ForegroundColor getForegroundColor() {
		synchronized (this.$lock_P) {
			return foreground;
		}
	}

    /**
     * Retrieves the background color.
     *
     * @return    the background color.
     */
    public BackgroundColor getBackgroundColor_internal() {
	return background;
    }

	/**
	 * Retrieves the background color.
	 * @return     the background color.
	 */
	public BackgroundColor getBackgroundColor() {
		synchronized (this.$lock_P) {
			return background;
		}
	}

    /**
     * Retrieves the attribute of this pen.
     *
     * @return    the attribute of this pen.
     */
    public Attribute getAttribute_internal() {
	return attribute;
    }

	/**
	 * Retrieves the attribute of this pen.
	 * @return     the attribute of this pen.
	 */
	public Attribute getAttribute() {
		synchronized (this.$lock_P) {
			return attribute;
		}
	}

    /**
     * Returns a string representation of <code>this</code>.
     * It has the form [ _foreground_ on _background_ with _attribute_ ].
     *
     * @return    the string representation of <code>this</code>.
     */
    public String toString_internal() {
	StringBuffer sb = new StringBuffer("[ ").append(foreground);
	sb.append(" on ").append(background).append(" with ").append(attribute);
	return sb.toString();
    }

	/**
	 * Returns a string representation of <code>this</code>. It has the form [ _foreground_ on _background_ with _attribute_ ].
	 * @return     the string representation of <code>this</code>.
	 */
	public String toString() {
		synchronized (this.$lock_P) {
			StringBuffer sb = new StringBuffer("[ ").append(foreground);
			sb.append(" on ").append(background).append(" with ").append(attribute);
			return sb.toString();
		}
	}

	protected final OrderedLock $lock_P;
}
