/*  Jcurzez - A free Java implementation of curses
 *  Copyright (C) 2001, 2006  Thomas G. Girard <thomas.g.girard@free.fr>
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.nongnu.savannah.jcurzez;

/**
 * A <b>Jcurzez</b> color, different from <code>java.awt.Color</code>.
 * There are two classes extending this one, respectively named
 * {@link BackgroundColor} and {@link ForegroundColor}. This one is package
 * private, it should not be used.
 * <p>
 * Basically, a <code>Color</code> is defined by its {@link #name name}
 * and by an object, its {@link #realColor real color}. This object will
 * depend on the implementation of <b>Jcurzez</b>.
 * Thus the user should not call {@link #setRealColor setRealColor(Object)}.
 * <p>
 * <i>Note: all colors are immutable ojects, so they can be compare using
 * <code>==</code>. Anyway, {@link #equals} is the same as <code>==</code>,
 * and it will remain ; I made it <code>final</code>.</i>
 *
 * @author Thomas Girard
 * @version 1.0.0
 */
class Color {

    /**
     * The real color object. Platform-dependent.
     */
    private         Object realColor;

    /**
     * Name of this color.
     */
    protected final String name;

    /**
     * Creates a new <code>Color</code> with specified name.
     *
     * @exception IllegalArgumentException thrown if <code>name</code> is
     *            <code>null</code>.
     */
    protected Color(String name) {
	if (name == null) {
	    throw new IllegalArgumentException("Color name must be non-null");
	}
	this.name = name;
    }

    /**
     * Returns the name of this color.
     *
     * @return {@link #name}.
     */
    public String toString() {
	return name;
    }

    /**
     * Modifies the real color. <b>This operation must be done only once,
     * (no more, no less) otherwise an runtime exception is thrown.</b>
     *
     * @param     o the object to store as a real color.
     * @exception IllegalColorStateException if this method is invoked
     *            more than once.
     * @exception IllegalArgumentException if <code>o</code> is
     *            <code>null</null>.
     */
    public void setRealColor(Object o) {
	if (realColor != null) {
	    throw new IllegalColorStateException("Color " + name +
						 " already has a real color");
	}
	if (o == null) {
	    throw new IllegalArgumentException("Color " + name +
					       "cannot have a null real value");
	}
	realColor = o;
    }

    /**
     * Returns the real color.
     *
     * @return {@link #realColor}.
     */
    public Object getRealColor() {
	return realColor;
    }

    /**
     * Tests equality. Prevents overridding by making it final.
     */
    public final boolean equals(Object o) {
	return super.equals(o);
    }
}
