/*  Jcurzez - A free Java implementation of curses
 *  Copyright (C) 2001, 2006  Thomas G. Girard <thomas.g.girard@free.fr>
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.nongnu.savannah.jcurzez;

/**
 * A <code>Pen</code> contains three attributes:
 * <ul>
 * <li>a {@link #foreground foreground color}</li>
 * <li>a {@link #background background color}</li>
 * <li>an {@link #attribute attribute}</li>
 * </ul>
 *
 * @author Thomas Girard
 * @version 1.0.0
 */
public class Pen {
    /** Foreground color for this <code>Pen</code>. */
    protected ForegroundColor foreground;

    /** Background color for this <code>Pen</code>. */
    protected BackgroundColor background;

    /** Attribute for this <code>Pen</code>. */
    protected Attribute attribute;
    
    AbstractWindow aw;

    /**
     * Creates a new pen with defaults values for
     * foreground color, background color and attribute.
     */
    public Pen() {
	this(ForegroundColor.WHITE, BackgroundColor.BLACK, Attribute.NORMAL);
    }

    /**
     * Creates a new pen with specified foreground, background
     * and attribute.
     *
     * @param     foreground the foreground color.
     * @param     background the background color.
     * @param     attribute the attribute fro this pen.
     */
    public Pen(ForegroundColor foreground,
	       BackgroundColor background,
	       Attribute attribute) {
	this.foreground = foreground;
	this.background = background;
	this.attribute = attribute;
    }

    /**
     * Checks whether <code>this</code> is equal to <code>o</code>.
     *
     * @param     o the object to compare <code>this</code> with.
     * @return    <code>true</true> if <code>this</code> is equal to
     *            <code>o</code>.
     */
    public boolean equals(Object o) {
	if (this == o)
	    return true;
	if (!(o instanceof Pen))
	    return false;
	Pen pen = (Pen) o;
	return (pen.foreground == foreground &&
		pen.background == background &&
		pen.attribute.equals(attribute));
    }

    /**
     * Changes the background color.
     *
     * @param     background the new background color.
     */
    public void setBackgroundColor(BackgroundColor background) {
    	synchronized(aw){
	this.background = background;
    	}
    }

    /**
     * Changes the foreground color.
     *
     * @param     foreground the new foreground color.
     */
    public void setForegroundColor(ForegroundColor foreground) {
    	synchronized(aw){
	this.foreground = foreground;
    	}
    }

    /**
     * Changes the attribute.
     *
     * @param     attribute the new attribute.
     */
    public void setAttribute(Attribute attribute) {
    	synchronized(aw){
	this.attribute = attribute;
    	}
    }

    /**
     * Retrieves the foreground color.
     *
     * @return    the foreground color.
     */
    public ForegroundColor getForegroundColor() {
	return foreground;
    }

    /**
     * Retrieves the background color.
     *
     * @return    the background color.
     */
    public BackgroundColor getBackgroundColor() {
	return background;
    }

    /**
     * Retrieves the attribute of this pen.
     *
     * @return    the attribute of this pen.
     */
    public Attribute getAttribute() {
	return attribute;
    }

    /**
     * Returns a string representation of <code>this</code>.
     * It has the form [ _foreground_ on _background_ with _attribute_ ].
     *
     * @return    the string representation of <code>this</code>.
     */
    public String toString() {
	StringBuffer sb = new StringBuffer("[ ").append(foreground);
	sb.append(" on ").append(background).append(" with ").append(attribute);
	return sb.toString();
    }
}
